function lambda_sparses = chooseSparsityParams(bigX, ones_to_use)
%Selects sparsity parameters for each tissue using method described in the paper: 
%select sparsity parameters for a subset of tissues using held-out log likelihood, then 
%select sparsity parameters for the remaining tissues by fitting a linear regression. 
%bigX: Cell array where each entry is data for one tissue, n_samples x n_genes. 
%ones_to_use: the indices of the elements in bigX to use for fitting lambda_sparse. 5 is probably sufficient. 
%lambda_sparses: the fitted lambda sparse values. 

n_to_fit = length(ones_to_use)
%1. Compute the optimal value of lambda sparse for each of the specified tissues. 
optimal_lambda_sparses = [];
for i = 1:n_to_fit
	optimal_lambda_sparses(end + 1) = chooseOptimalLambdaSparse(bigX{ones_to_use(i)});
end
%2. Use fitted values of lambda sparse to choose for remaining matrices. 
lambda_sparses = fitLambdaSparse(bigX, ones_to_use, optimal_lambda_sparses);
end

function lambda_sparses = fitLambdaSparse(bigX, ones_to_use, optimal_lambda_sparses)
%bigX: Cell array where each entry is data for one tissue, n_samples x n_genes. 
%ones_to_use: indices of matrices in bigX for which optimal_lambda_sparses were computed. 
%lambda_sparses: all fitted values of lambda sparse.
GLASSO_PATH = 'YOUR_GLASSO_DIRECTORY_PATH';%change this to appropriate path. 
addpath(GLASSO_PATH); 
lambda_sparses = [];
n_tissues = length(bigX);
heuristic_values = [];
%First compute the heuristic we use to predict lambda sparse. 
for i = 1:n_tissues 
	[n_samples, n_genes] = size(bigX{i});
	empCorr = corr(bigX{i});
	assert(size(empCorr, 1) == n_genes)
	heuristic_values(end + 1) = log(n_samples) / mean(abs(empCorr(:))); 
end
%Now run a regression to fit the heuristic to the values chosen by cross-validation. 
M = [ones(1, n_tissues); heuristic_values]';
[b,bint,r,rint,stats] = regress(optimal_lambda_sparses',M(ones_to_use, :));
fprintf('The correlation between fitted values and actual values: %2.3f\n', stats(1))
%Use regression to predict remaining values. 
lambda_sparses = M*b;
lambda_sparses(ones_to_use) = optimal_lambda_sparses;

end

function optimal_lambda = chooseOptimalLambdaSparse(X)
%Given a dataset, uses cross validation to choose optimal lambda sparse value. 
%X: an n x p matrix
%optimal_lambda: the lambda with the highest score using held-out cross validation. 
k = 5;
n_lambdas = 20; 
lambdaList = (1/n_lambdas):(1 / n_lambdas):1;
approximate = 0;
verbose = 0;
penalDiag = 0;
[n_samples, n_genes] = size(X);	
cross_val_indices = crossvalind('Kfold', n_samples, k);
held_out_lls = zeros(k, n_lambdas);
for i = 1:k % for each cross validation fold, test a range of lambda sparse values. 
	test = (cross_val_indices == i);
	train = (cross_val_indices ~= i);
	trainX = makeVarianceOne(X(train, :));
	testX = makeVarianceOne(X(test, :));
	assert((size(trainX, 2) == n_genes) && (size(trainX, 1) == sum(train))); 
	[wList, thetaList, lambdasTested, errors] = GraphicalLassoPath(trainX, lambdaList, approximate, verbose, penalDiag);
	testSigma = corr(testX);
	assert(sum(abs(lambdasTested - lambdaList)) < 1e-6)
	for j = 1:n_lambdas
		held_out_lls(i, j) = gaussianLL(thetaList(:, :, j), testSigma, sum(test));
		fprintf('Matrix %i has %i nonzero off-diagonal elements and likelihood %2.3f\n', j, nnz(thetaList(:, :, j)) - n_genes, held_out_lls(i, j))
	end
end
held_out_lls = mean(held_out_lls, 1);
[max_ll, max_idx] = max(held_out_lls);
optimal_lambda = lambdaList(max_idx) * n_samples;
fprintf('Best value of lambda: %2.3f yielding likelihood %2.3f\n', optimal_lambda, max_ll);
end


function ll=gaussianLL(K, emp_sigma, n)
%computes the Gaussian likelihood. 
%K: the precision matrix
%emp_sigma: the empirical covariance matrix. 
%n: the number of samples
%ll: the log likelihood. 
if isnan(emp_sigma)
	ll=-Inf;
else
	d=size(K, 1);
	logDetTerm=log(det(K))*n/2;
	fro_pro_Term=-fro_prod(K, emp_sigma)*n/2;
	constantTerm=-n*d*log(2*pi)/2;
	ll=constantTerm+logDetTerm+fro_pro_Term;
end
assert(~isnan(ll));
end

function s=fro_prod(A, B)
s=sum(sum(A'.*B));
end


function X = makeVarianceOne(X)
%Ensures that each column of X has variance 1 (so that cov = corr). 
oldMeans=mean(X, 1);
[n_samples, c]=size(X);
stds=std(X, 0, 1);

stds(abs(stds)<1e-6)=1;
mean_matrix=repmat(mean(X), n_samples, 1);
std_matrix=repmat(stds, n_samples, 1);
X = mean_matrix+(X-mean_matrix)./std_matrix;
assert(sum(abs(mean(X)-oldMeans))<1e-6); 
end


