function optimizedMatrix = optimizeChild(empCov, n_samples, parentCov, x0, lambda_parent, lambda_sparse, component_idxs)
%Optimizes a single external node. 
%empCov: empirical correlation matrix for the tissue. 
%n_samples: number of samples for tissue. 
%parentCov: optimized network for the parent of the tissue. 
%x0: the initialization for the optimization algorithm. 
%lambda_parent: the L2 parent similarity penalty. 
%lambda_sparse: the L1 sparsity penalty. 
%component_idxs: the indices of the block diagonal components. 
%optimizedMatrix: the optimized network. 
L1Path = 'YOUR_L1_DIRECTORY_PATH';% replace this with directory where you installed L1General
addpath(genpath(L1Path))
n_genes=size(empCov, 1);
assert(size(parentCov, 1) == n_genes);
assert(size(parentCov, 2) == n_genes);
assert(length(component_idxs) == n_genes);
component_values = unique(component_idxs);

if lambda_parent > 0 %normalize parent matrix so it's on the same scale as child. 
	normalizedParentCov = normalizeParentCov(parentCov, x0); 
else
	normalizedParentCov = parentCov;
end

optimizedMatrix = zeros(n_genes, n_genes);
assert(isSym(parentCov) == 1);
assert(isSym(x0) == 1);

for i = 1:length(component_values)%optimize each block-diagonal component separately
	this_component = (component_idxs == (component_values(i)));
	p = sum(this_component);
	if p == 1 %a singleton element -- just set to 1. 
		optimizedMatrix(this_component, this_component) = 1;
	else %otherwise, optimize objective using L1 package. 
		upper_idxs = find(triu(ones(p))); %only optimize triangle of matrix (b/c symmetric). 
		lambda_sparse_matrix=lambda_sparse*ones(p);
		lambda_sparse_matrix(eye(p)==1)=.0001*lambda_sparse;%slight diagonal penalty.
		lambda_sparse_matrix=lambda_sparse_matrix(upper_idxs);
		small_x0 = x0(this_component, this_component);
		small_x0 = small_x0(upper_idxs);
		small_normalizedParentCov = normalizedParentCov(this_component, this_component);
		small_empCov = empCov(this_component, this_component);
		assert(numel(small_x0) == (p * (p + 1) / 2));
		assert(numel(small_normalizedParentCov) == p^2);
		assert(numel(small_empCov) == p^2);

		objFun = @(x) l2PenalizedLL(x, small_empCov, small_normalizedParentCov,lambda_parent,n_samples);
		optval = L1General2_PSSas(objFun, small_x0, lambda_sparse_matrix);
		M = zeros(p, p);
		M(upper_idxs) = optval;
		M = (M + M')/2;%ensure symmetric.
		optimizedMatrix(this_component, this_component) = M;
	end
end

end

function normalizedParentCov = normalizeParentCov(parentCov, oldChild)
%This ensures that the parent and child matrices are on the same scale by normalizing the parent
%Specifically, they have same average absolute values of on-diagonal and off-diagonal elements. 
%Ensures we are comparing apples to apples when using parent penalty. 
%parentCov: the (unnormalized) optimized network for the parent. 
%oldChild: the optimized network for the child from the previous iteration (used to determine scale of child)
%normalizedParentCov: the (normalized) optimized network for the parent. 

n_genes = size(oldChild, 1);
d=find(eye(n_genes));
off_d=find(eye(n_genes)==0);
p_d=full(mean(abs(parentCov(d))));
off_d_idxs=setdiff(find(oldChild), d);
parent_off_d_idxs=setdiff(find(parentCov), d);
p_off_d=full(mean(abs(parentCov(off_d_idxs))));
c_d=full(mean(abs(oldChild(d))));
c_off_d=full(mean(abs(oldChild(off_d_idxs))));
if p_d>0
	parentCov(d)=parentCov(d)*c_d/p_d;
end
if p_off_d>0
	parentCov(parent_off_d_idxs)=parentCov(parent_off_d_idxs)*c_off_d/p_off_d;
end
normalizedParentCov = parentCov;
end

function [fval, grad]=l2PenalizedLL(K, emp_sigma, parent_K, lambda_parent, n_samples)
%Computes objective and gradient for l2-penalized log likelihood.
%K: the upper triangle of the precision matrix. 
%emp_sigma: the empirical covariance. 
%parent_K: optimized network for the parent of the tissue. 
%lambda_parent: the L2 parent similarity penalty. 
%n_samples: the number of samples. 
%fval: the value of the objective function. 
%grad: the gradient of the objective function. 

assert(isempty(find(isinf(K))) && isempty(find(isnan(K))))
n=size(emp_sigma, 1);
upper_idxs = find(triu(ones(n)));
matrixK = zeros(n, n);
matrixK(upper_idxs) = K;
K=(matrixK + matrixK')/2;
assert(isSym(K) == 1);
assert(isSym(parent_K) == 1);
diff=K-parent_K;
n_genes=size(K, 1);
log_det=full(fastLogDet(K));
trace_term=full(n_samples*fro_prod(K, emp_sigma));
parent_difference=full(lambda_parent*sum(sum(diff.^2)));
fval=-n_samples*log_det+trace_term+parent_difference;
grad=-n_samples*inv(K)+n_samples*emp_sigma+2*lambda_parent*diff;
grad = grad(upper_idxs);
assert(sum(isnan(grad))==0)
assert(~isnan(fval))
end

function sym = isSym(X)
%tests whether X is symmetric. 
if sum(sum(abs(X - X'))) < 1e-6
	sym = 1;
else
	sym = 0;
end
end

function d=fastLogDet(X)
%computes log determinant quickly using Cholesky decomposition. 
[R, p]=chol(X);
if p==0
  d=2*sum(log(diag(R)));
else
  d=-Inf;
end
end


function s=fro_prod(A, B)
%computes Frobenius product. 
s=sum(sum(A'.*B));
end
