function networks = optimizeNetworks(bigX, lambda_sparses, lambda_parent, max_comp_size, verbose)
%Main function. Optimizes networks using method described in paper. 
%bigX: Cell array where each entry is data for one tissue, n_samples x n_genes. 
%lambda_sparses: n_tissues x 1 vector of sparsity parameters for each tissue. 
%lambda_parent: value of the parent penalty. 
%max_comp_size: optional argument; restricts maximum size of component in network for 
%computational efficiency. Set to n_genes by default.
%verbose: if 1, prints out output .

if size(bigX, 1) == 1
	bigX = bigX';
end
converged = 0;
n_iter = 0;
max_iter = 50;
n_tissues = length(bigX);
n_genes = size(bigX{1}, 2);
n_samples = cellfun(@(X_i) size(X_i, 1), bigX);
if nargin < 4
	max_comp_size = n_genes;
end
if nargin < 5
	verbose = 0;
end
if verbose
	fprintf('Number of tissues: %i; number of genes: %i\n', n_tissues, n_genes);
end
J = n_tissues * 2 - 1;
tissue_hierarchy=clusterTissues(bigX);%calculate tissue hierarchy. 
if max_comp_size < n_genes%if desired, use block-diagonal approximation. 
	component_idxs = calcConnComp(bigX, lambda_sparses, max_comp_size, verbose);
else
	component_idxs = ones(n_genes, 1);
end

networks = cell(J, 1);
empCovs = cell(J, 1);
for j = 1:J
	networks{j} = eye(n_genes);
	if j <= n_tissues
		empCovs{j} = corr(bigX{j});
	end
end

while (~converged) && (n_iter < max_iter)%main loop. 
	n_iter = n_iter + 1;
	old_networks = networks;
	if verbose
		fprintf('Iteration %i; optimizing children.\n', n_iter)
	end
	for j = 1:n_tissues %compute external nodes. This loop can be parallelized. 
		parent_idx = findParentIdx(tissue_hierarchy, j);
		if n_iter == 0 % for first iteration, initialize using standard graphical lasso. 
			assert(size(bigX{j}, 2) == n_genes);
			networks{j} = optimizeChild(empCovs{j}, n_samples(j), networks{parent_idx}, networks{j}, 0, lambda_sparses(j), component_idxs);
		else % otherwise, optimize objective in paper for a single external node. 
			networks{j} = optimizeChild(empCovs{j}, n_samples(j), networks{parent_idx}, networks{j}, lambda_parent, lambda_sparses(j), component_idxs);
		end
		if verbose
			fprintf('Optimized child %i; number of nonzero entries, %i\n', j, nnz(networks{j}))
		end
	end
	%optimize internal nodes
	networks((n_tissues + 1):end) = optimizeParents(networks(1:n_tissues), tissue_hierarchy);
	converged = checkConvergence(old_networks(1:n_tissues), networks(1:n_tissues), verbose);
	if converged == 1
		fprintf('Algorithm converged after %i iterations\n', n_iter)
	end
	if n_iter >= max_iter
		fprintf('Maximum number of iterations (%i) has been reached\n', max_iter)
	end
end
networks = networks(1:n_tissues);
end

function converged = checkConvergence(old_networks, networks, verbose)
%Checks whether change in external networks is less than a certain tolerance. 
%old_networks: cell array of external networks from last iteration. 
%networks: cell array of external networks from current iteration. 
tol=1e-4;
converged=1;
n_tissues = length(networks);
assert(length(networks) == length(old_networks));
for i=1:n_tissues
	change=full(sum(sum((old_networks{i}-networks{i}).^2)));
    relative_change=full(change/sum(sum((networks{i}.^2))));
    if verbose
   		fprintf('Checking convergence in network %i; relative change %2.3e, threshold %2.3e\n', relative_change, tol)
   	end
    if relative_change>tol
     	converged=0;
    end
end

end

function parent_idx = findParentIdx(tissue_hierarchy, j)
%Small helper method. 
%Finds the parent_idx of the parent of j given the tissue_hierarchy.
n_tissues = size(tissue_hierarchy, 1) + 1;
for i = 1:(n_tissues - 1)
	if (tissue_hierarchy(i, 1) == j) || (tissue_hierarchy(i, 2) == j)
		parent_idx = i + n_tissues;
	end
end
end

function tissue_hierarchy = clusterTissues(bigX)
%bigX: Cell array where each entry is data for one tissue, n_samples x n_genes. 
%tissue_hierarchy: the hierarchy of tissues used for clustering. 
n_genes = size(bigX{1}, 2);
n_tissues = length(bigX);
tissue_means = zeros(n_tissues, n_genes);
for i = 1:n_tissues
	tissue_means(i, :) = mean(bigX{i}, 1);
end
tissue_hierarchy=linkage(tissue_means);
assert(size(tissue_hierarchy, 1) + 1 == length(bigX))
end

function idxs = calcConnComp(bigX, lambda_sparses, max_comp_size, verbose)
%If block diagonal approximation is desired, this function computes indices for the blocks.
%Extends methods from Hsieh et al, 2012 and Witten et al, 2011. 
%bigX: Cell array where each entry is data for one tissue, n_samples x n_genes. 
%lambda_sparses: n_tissues x 1 vector of sparsity parameters for each tissue. 
%max_comp_size: the maximum allowable component size in the network. 
assert((max_comp_size > 0) && (mod(max_comp_size, 1) == 0))
n_tissues = length(bigX);
n_genes = size(bigX{1}, 2);
S=sparse(zeros(n_genes));%compute the matrix S used to approximate clustering. 
if max_comp_size == 1
	idxs = 1:n_genes;
	return;
end
for j = 1:n_tissues
	n_samples = size(bigX{j}, 1);
	c = corr(bigX{j});
	assert(size(c, 1) == n_genes);
	c=sparse(max(c-lambda_sparses(j) / n_samples, 0).^2);
	S=S+c*n_samples;
end
[n_comps, idxs] = partitionS(S, max_comp_size, verbose); % compute the clustering given S.
end

function [n_comps, C]=partitionS(S, max_comp_size, verbose) 
%S: the sum of covariance matrices thresholded by lambda_sparse as computed in calcConnComp
%max_comp_size: the maximum allowable component size in the network. 
%n_comps: the number of components the matrix has been partitioned into. 
%C: the partition. 
p=size(S, 1);
wittenG=(S~=0);
d=find(eye(p)==1);
off_d=find(eye(p)==0);
nonzeros=nnz(wittenG(off_d));
if verbose
	fprintf('Number of nonzero entries: %i\n', nonzeros)
end
if nonzeros==0
	C=1:p;
	n_comps=p;
else
	%remove nodes which are completely detached from all other nodes.
	C=zeros(p, 1);	
	[witten_n_groups, wittenC]=graphconncomp(wittenG);
	counts=histc(wittenC, 1:witten_n_groups);
	clustersWithOnlyOne=find(counts==1);
	singletonIdxs=find(arrayfun(@(i) ismember(i, clustersWithOnlyOne), wittenC));
	nonsingletonIdxs=setdiff(1:p, singletonIdxs);
	n_singletons=length(singletonIdxs);
	if verbose
		fprintf('Removing %i singletons...\n', n_singletons)
	end
	C(singletonIdxs)=1:n_singletons;
	if p-n_singletons<=max_comp_size%if after we remove singletons, component size is acceptable, stop. 
		C(nonsingletonIdxs)=n_singletons+1;
		if verbose
			fprintf('Terminating; component size %i, after removing %i singletons, max component %i\n', p, n_singletons, p-n_singletons);	
		end
	else%otherwise, split the remaining nodes into two groups using method in Hsieh et al, 2012, and repeat. 	        
		remainderS=S(nonsingletonIdxs, nonsingletonIdxs);
		[r1, r2]=bipartition(remainderS, verbose);
		if verbose
			fprintf('Further partitioning nonsingleton component into two components of size %i and %i\n',length(r1), length(r2));
		end
		if length(r1)<=max_comp_size
			C1=ones(length(r1), 1);
		else
			[~, C1]=partitionS(remainderS(r1, r1), max_comp_size, verbose);
		end
		if length(r2)<=max_comp_size
			C2=ones(length(r2), 1);
		else
			[~, C2]=partitionS(remainderS(r2, r2), max_comp_size, verbose);
		end
		n1=length(unique(C1));
		C(nonsingletonIdxs(r1))=n_singletons+C1;
		C(nonsingletonIdxs(r2))=n_singletons+n1+C2; 
	end
	[vals, ~, C]=unique(C);
	n_comps=length(vals);
end
end

function [best_idxs1, best_idxs2]=bipartition(S, verbose)
%Partitions S into two groups using eigenvectors following Hsieh et al, 2012
S=sparse(S);
d=sum(S, 1);
p=size(S, 1);
D=sparse(diag(d));
invSqrt=diag(d.^-.5);
invSqrt(isinf(invSqrt))=0;
M=sparse(invSqrt*(D-S)*invSqrt);
assert(sum(sum(abs(M-M')))<1e-8)
M=(M+M')/2;
if verbose
	fprintf('Computing eigenvectors; number of nonzero elements in matrix is %i\n', nnz(M));
end
opts.issym=1;
[V, eigenvalues]=eigs(M, 2, 'SA', opts);
v=V(:, 2);
sorted_v=sort(v);
l=sorted_v(floor(p/10)+1);
u=sorted_v(floor(9*p/10));
split_points=l+(1:49)*(u-l)/50;%search over 50 possible split points. 
if verbose
	fprintf('Smallest eigenvector entry: %f; largest eigenvector entry: %f\n', l, u);
end
bestV=Inf;
for i=1:length(split_points)
	idxs1=find(v<split_points(i));
	idxs2=setdiff(1:p,idxs1);
	val=full(sum(sum(S(idxs1, idxs2)))/sum(d(idxs1))+sum(sum(S(idxs2, idxs1)))/sum(d(idxs2)));
	if val<bestV
		bestV=val;
		best_idxs1=idxs1;
		best_idxs2=idxs2;
	end	
end
end

