function parents = optimizeParents(leaves, tissue_hierarchy)
%Optimizes the internal nodes given the networks for the external nodes. 
%leaves: a cell array of networks for external nodes. 
%tissue_hierarchy: a hierarchy of tissues as returned by Matlab's linkage function. 
%parents: a cell array of networks for the internal nodes. 
%Note: if desired, the weight function (line 84) can be set to something other than 1: 
%this will place stronger similarity constraints on nodes that are lower in the hierarchy 
%implying that they may be more similar. 

% First, create the matrices needed to analytically optimize internal L2 penalty. 
%y=-A^(-1)Bx where x is external nodes, y internal nodes. 
[A, B]=makeMatrices(tissue_hierarchy); 
nodes={};
n_tissues=size(A, 1)+1;
n_genes=size(leaves{1}, 1);
%Normalize the leaves so they are on the same scale and can be compared. 
%Each matrix has diagonal + off-diagonal elements mean 1.
diagonal_idx=find(eye(n_genes));
all_nnzs=[];
for i=1:n_tissues 
	mean_diag=full(mean(abs(leaves{i}(diagonal_idx))));
	nnzs=find(leaves{i});
	offdiag_nnzs=setdiff(nnzs, diagonal_idx);
	all_nnzs=union(all_nnzs, nnzs);
	mean_offdiag_nnzs=full(mean(abs(leaves{i}(offdiag_nnzs))));
	leaves{i}(diagonal_idx)=leaves{i}(diagonal_idx)/mean_diag;
	leaves{i}(offdiag_nnzs)=leaves{i}(offdiag_nnzs)/mean_offdiag_nnzs;
end
%Now compute optimal value for internal nodes. 
%First put all nonzero entries in external nodes into a single matrix .
m=length(all_nnzs);
x=zeros(n_tissues, m);
for i=1:n_tissues
	   x(i, :)=leaves{i}(all_nnzs);
end
%Analytically optimize internal nodes for nonzero elements. 
optimizedParents=-A\B*x;
parents=cell(n_tissues-1, 1);
for i=1:n_tissues-1
	parents{i}=sparse(zeros(n_genes));
	parents{i}(all_nnzs)=optimizedParents(i, :);	
end

end

function[A, B]=makeMatrices(tissue_hierarchy)
%Creates the matrices needed to analytically optimize the internal L2 penalty. 
%A and B are matrices such that Ay+Bx=0 yields the correct solution for y! 
%y is the optimized parent vector (n_internal x 1) and x is the leaf vector (n_tissues x 1)
%A is n_internal x n_internal
%B is n_internal by n_tissues
n_internal=size(tissue_hierarchy, 1);
n_tissues=n_internal+1;
rootHeight=tissue_hierarchy(n_internal, 3);
A=zeros(n_internal, n_internal);
B=zeros(n_internal, n_tissues);
for i=1:n_internal
	height=tissue_hierarchy(i, 3);
	f=height/rootHeight;
	weight=weightFn(f); 
	%if desired, give larger weights to similarities between nodes lower in hierarchy. 
	c1=tissue_hierarchy(i, 1);
	c2=tissue_hierarchy(i, 2);
	if c1<=n_tissues
		B(i, c1)=-weight;
	else
		A(i, c1-n_tissues)=-weight;
		A(c1-n_tissues, i)=-weight;
		A(c1-n_tissues, c1-n_tissues)=A(c1-n_tissues, c1-n_tissues)+weight;
	end
	if c2<=n_tissues
		B(i, c2)=-weight;
	else
		A(i, c2-n_tissues)=-weight;
		A(c2-n_tissues, i)=-weight;
		A(c2-n_tissues, c2-n_tissues)=A(c2-n_tissues, c2-n_tissues)+weight;
	end
	A(i, i)=A(i, i)+2*weight;
end


end

function y=weightFn(x)
%Weight by height of the node in the hierarchy. 
%Set to 1 by default, but could set it to something else -- eg, 1/(a+x). 
y=1;
end


