function [bigX, gene_names]=processRawData(bigX, n_genes_to_take, gene_names, GO_gene_names)
%Normalizes read count data, projects it onto a Gaussian, and selects tissue-specific genes.
%bigX: Cell array where each entry is read count data for one tissue, n_samples x n_genes. 
%n_genes_to_take: the number of genes to select. 
%gene_names: the names of the genes in bigX. 
%GO_gene_names: the names of the genes in Gene Ontology. 
%bigX: the processed data. 
%gene_names: the names of the selected genes. 

if size(bigX, 1) == 1
	bigX=bigX';
end

J=length(bigX);
n_genes=length(gene_names);
genes_selected=[];
all_zeros=[]; 

%%First, filter out genes that are too frequently zero (or all constant)
%%take log of readcounts, project onto Gaussian, make variance of each gene in each tissue 1. 
for j=1:J
	[n, m]=size(bigX{j});
	mostFrequentValue=mode(bigX{j});
	minDiff=floor(n/5)+1;%require that genes differ from mode value in at least 1/5 of samples. 
	numDifferentFromMode=sum((bigX{j}-repmat(mostFrequentValue, n, 1))~=0, 1);
	all_constant=find(numDifferentFromMode<=minDiff);
	all_zeros=unique([all_zeros, all_constant]);
	bigX{j}=5e7*bigX{j}./repmat(sum(bigX{j}, 2), 1, m);%%Divide by total number of reads.    
	bigX{j}=log(2+bigX{j})/log(2);%%Take the log base two.
	for i=1:size(bigX{j}, 2)
		bigX{j}(:, i)=gaussianProject(bigX{j}(:, i));%%project onto Gaussian for each tissue. 
	end
	bigX{j}=makeVarianceOne(bigX{j});%%standardize the variance for each tissue. 
end

%%Now compute ranked list of most highly expressed genes for each tissue.
fullX=cell2mat(bigX);
fullMean=mean(fullX, 1);%%mean of each gene across all samples
fullStds=std(fullX, 1);%%standard deviation of each gene across all samples.
rankings = zeros(J, n_genes);
for j=1:J
	%%rank genes by how much expression in tissue differs from average expression (normalizing by variance) 
	quad_stds=sqrt(fullStds.^2+std(bigX{j}, 1).^2);
	quad_stds(quad_stds==0)=1;
	mean_normalized_reads=(mean(bigX{j}, 1)-fullMean)./quad_stds;
	[num, tissue_rankings]=sort(mean_normalized_reads, 'descend');
	rankings(j, :)=tissue_rankings; 
end

%%Now loop over tissues until you have enough genes, selecting the gene that is most highly
%%expressed in each tissue, is not constant or all zero in any tissue, and is included in GO. 
i=1;
idxs=ones(J, 1);
while length(genes_selected)<n_genes_to_take
	next_gene=rankings(i, idxs(i));
	next_gene_name = gene_names(next_gene);
	while ismember(next_gene, genes_selected) || ismember(next_gene, all_zeros) || (~ismember(next_gene_name, GO_gene_names))
		idxs(i)=idxs(i)+1;
		next_gene=rankings(i, idxs(i));
		next_gene_name = gene_names(next_gene);
	end
	idxs(i)=idxs(i)+1;
	genes_selected=[genes_selected, next_gene];
	i=i+1;
	if i>J
		i=1;
	end
end
%%Make sure there are no genes with duplicate names. 
[gene_names, unique_idx]=unique(gene_names(genes_selected));
genes_selected=genes_selected(unique_idx);
for j=1:J
	bigX{j}=bigX{j}(:, genes_selected);
end

end

function v=gaussianProject(x)
%%projects x onto a Gaussian v. 
p=percentile(x);
p(p==1)=.99;%otherwise these values get sent to infinity. 
mu=mean(x);
sigma=std(x);
if sigma==0
	sigma=1;
end
v=norminv(p, mu, sigma);
end


function X = makeVarianceOne(X)
%Ensures that each column of X has variance 1. 
oldMeans=mean(X, 1);
[n_samples, c]=size(X);
stds=std(X, 0, 1);
stds(abs(stds)<1e-6)=1;
mean_matrix=repmat(mean(X), n_samples, 1);
std_matrix=repmat(stds, n_samples, 1);
X = mean_matrix+(X-mean_matrix)./std_matrix;
assert(sum(abs(mean(X)-oldMeans))<1e-6); 
end
