function [] = testAlgorithm()
%Tests algorithm on simulated networks. 
%If networks are totally unrelated, lambda_parent > 0 doesn't improve accuracy. 
%But for related networks, lambda_parent > 0 does. 

rng(23)
n_genes = 20;
nonzero_frac = .05;
offdiag_size = .3;
n_tissues = 2;
n_samples = [200, 200];
lambda_parents = [0, 10, 20, 50, 100, 200, 300]; %range of lambda_parent values to test. 
lambda_sparses = [20, 20]; %lambda_sparse for each network
verbose = 0;
max_comp_size = 20;

for sim_param = [0, .5] % first generate unrelated networks; then generate related networks. 
	[bigX, precisions, relationships] = generateFakeData(n_genes, sim_param, nonzero_frac,offdiag_size, n_samples);
	overlap = nnz((precisions{1} ~= 0) & (precisions{2} ~= 0));
	fprintf('\n\nGenerated data: NNZ in 1: %3i, NNZ in 2: %3i; NNZ in both: %3i, similarity: %2.3f\n', nnz(precisions{1}), nnz(precisions{2}), overlap, sim_param)
	for lambda_parent = lambda_parents
		optimizedNetworks = optimizeNetworks(bigX, lambda_sparses, lambda_parent, max_comp_size);
		[f1, precision, recall] = computeF1Score(optimizedNetworks, precisions);
		nnz1 = nnz(optimizedNetworks{1});
		nnz2 = nnz(optimizedNetworks{2});
		nnzboth = nnz((optimizedNetworks{2} ~= 0) & (optimizedNetworks{1} ~= 0));
		correct1 = nnz((optimizedNetworks{1} ~=0) & (precisions{1} ~= 0));
		correct2 = nnz((optimizedNetworks{2} ~=0) & (precisions{2} ~= 0));
		fprintf('Links shared between both networks: %3i; l_p: %2.3f; F1: %2.3f Precision: %2.3f Recall: %2.3f\n', nnzboth, lambda_parent, f1, precision, recall);
	end
end

end

function m = ROC(m1, m2)
%Computes ROC of predicted vs actual sparsity patterns. 
%m1: predicted sparse matrix
%m2: actual sparse matrix
%m: ROC matrix.

p=size(m1, 1);
n=p*p;
m1(eye(p)==1)=0;
m2(eye(p)==1)=0;
i1=find(abs(m1)>1e-6);
i2=find(abs(m2)>1e-6);
true_p=length(intersect(i1, i2));
false_p=length(setdiff(i1, i2));
false_n=length(intersect(setdiff(1:n, i1), i2));
true_n=n- p - true_p-false_p-false_n;
m=[true_p, false_p; false_n, true_n]/(n - p);

end

function [f1, precision, recall]=computeF1Score(Ss, precisions)
%Computes the average precision, recall, and F1 score (across tissues) of the off-diagonal nonzero links.
%Ss: cell array of predicted sparse precision matrices (one for each tissue)
%precisions: cell array of true sparse precision matrices
%f1: average F1 score across tissues. 
%precision: average precision across tissues. 
%recall: average recall across tissues. 
 
J=length(Ss);
f1s=[];
precision_scores = [];
recall_scores = [];
d=size(Ss{1}, 1);
for j=1:J
	ROCmatrix=ROC(Ss{j}, precisions{j});
	f1s(end+1)=2*ROCmatrix(1, 1)/(2*ROCmatrix(1, 1)+ROCmatrix(2, 1)+ROCmatrix(1, 2));
	precision_scores(end + 1) = ROCmatrix(1, 1) / (ROCmatrix(1, 1) + ROCmatrix(1, 2));
	recall_scores(end + 1) = ROCmatrix(1, 1) / (ROCmatrix(1, 1) + ROCmatrix(2, 1));
end
f1=mean(f1s);
precision = mean(precision_scores);
recall = mean(recall_scores);
end



function [X, precisions, relationships]=generateFakeData(n_genes, sim_param, nonzero_frac, offdiag_size, n_samples)
%Draws data from Gaussians for two related, simulated tissues. 
%n_genes: Number of genes. 
%sim_param: controls similarity of Gaussian precision matrices become more similar.
%nonzero_frac: controls number of nonzero elements in precision matrices. 
%offdiag_size: controls size of offdiagonal elelemnts in precision matrices. 
%n_samples: vector with the number of samples for each tissue. 

n_tissues = 2;
relationships=[1, 0; sim_param, 1-sim_param];
precisions=cell(n_tissues, 1);
linComb=rand(n_genes, n_tissues);
basePrecisions=cell(n_tissues, 1);
precisions={};
X={};
negative_frac=.9;
base_sparsities={};
for i=1:n_tissues
	baseMeans{i}=linComb*relationships(i, :)'/sum(relationships(i, :));
	sparsity_pattern=rand(n_genes)>(1-nonzero_frac);
	sparsity_pattern=sparsity_pattern+sparsity_pattern';
	sparsity_pattern(eye(n_genes)==1)=1;
	off_diag=find(eye(n_genes)==0);
	c=rand(n_genes);
	c=c+c';
	c(off_diag)=(c(off_diag)-negative_frac)*offdiag_size;
	c(eye(n_genes)==1)=1;
	c=c.*sparsity_pattern;
	base_sparsities{i}=find(abs(c)>1e-6);
	basePrecisions{i}=c;
end


for i=1:n_tissues
    w=relationships(i, :)/sum(relationships(i, :));
	precisions{i}=diag(diag(basePrecisions{1}));
	for j=1:length(w)
		n_on=length(base_sparsities{j});
		ones_to_turn_on=find(rand(n_on, 1)<w(j));
        precisions{i}(base_sparsities{j}(ones_to_turn_on))=basePrecisions{j}(base_sparsities{j}(ones_to_turn_on));
    end
	precisions{i}=precisions{i}+precisions{i}';
	[L, p]=chol(precisions{i});
	while p~=0
        precisions{i}(eye(n_genes)==1)=precisions{i}(eye(n_genes)==1)+.2;
        [L, p]=chol(precisions{i});
        fprintf('Matrix non-PSD; adding to diagonal.\n');
    end		
	X{i}=mvnrnd(zeros(n_genes, 1), inv(precisions{i}), n_samples(i));
end
end

